.nolist	; optional - prevents a long listing of the includes

include \masm32\include\masm32rt.inc	; include Masm32 standard stuff
; 
; global variables and procedures
; -------------------------------
include     LibModA.glb
include     LibModB.glb
include     LibModC.glb

includelib  MyLib1.lib		; the library offering bits and pieces of code to the linker
; 
COMMENT @

This is a complete example how you can build a library in Masm.

The package contains the following files:

TestMyLib.asm	= the file you are reading right now
makeMyLib.bat	= batch file that builds the library from its components/modules
LibModA.asm	= first module, contains the ShowBoxA procedure
LibModB.asm	= second module, contains the ShowBoxB procedure
LibModC.asm	= third module, contains the ShowBoxC procedure
MyLib.inc	= include file, contains info for the linker as shown below

Included but not needed because you should create them yourself:

mylib.lib	= the library itself, as built by makeMyLib.bat
TestMyLib.exe	= the executable created with the file you are reading right now.

NOTE: EXTERNDEF is case-insensitive; however, to facilitate the understanding,
I have put it in all UPPERCASE where the variable is being created and EXported,
and mixed case (ExternDef) where the variable is being used and IMported.

Contents of MyLib.inc:
ShowBoxA	PROTO: DWORD	; the Protos inform the linker about the arguments for each
ShowBoxB	PROTO: DWORD	; procedure - in this case, a dword for passing a pointer to
ShowBoxC	PROTO: DWORD	; a string. No need to define them in the source modules.

ExternDef	SharedVar:DWORD		; these are the variables (structures, labels, ...) defined
ExternDef	hWin:DWORD			; somewhere in one of the modules but used in common.
ExternDef	MyLabel:NEAR

End COMMENT @

.code
MyTxt	db "pretty useless string", 0

start:	invoke ShowBoxA, addr MyTxt

  ; str$ and chr$ are macros coming with masm32rt.inc
  ; SharedVar is a common variable declared as follows:
  ; - declared and defined for export in LibModA.asm as "EXTERNDEF SharedVar:DWORD"
  ; - declared for importing by the calling procedure, i.e. here, in MyLib.inc - see above
  ; the ShowBoxA procedure in LibModA.asm returns the clicked value as mov SharedVar, eax

	.if SharedVar == IDYES
		invoke MessageBox, hWin, chr$("YES"),
		chr$("Clicked in ShowBoxA:"), MB_OK

	.elseif SharedVar == IDNO
		invoke MessageBox, hWin, chr$("NO"),
		chr$("Clicked in ShowBoxA:"), MB_OK

	.elseif SharedVar == IDCANCEL
		invoke MessageBox, hWin, chr$("CANCEL"),
		chr$("Clicked in ShowBoxA:"), MB_OK

	.else
		invoke MessageBox, hWin, chr$("What the hell did you click??"),
		chr$("Clicked in ShowBoxA:"), MB_OK
	.endif

	invoke ShowBoxB, addr MyTxt

  ; adding ShowBoxC will insert a new routine into the code (but you need OllyDbg to see it):
	; invoke ShowBoxC, addr MyTxt

  ; just for fun, here an example how to use an externally defined label:
	mov eax, offset MyLabel
	sub eax, offset ShowBoxA
	invoke MessageBox, hWin, str$(eax),
	chr$("The length in bytes of the ShowBoxA procedure:"), MB_OK
	
	exit		; aka invoke ExitProcess, 0 - this is a macro provided by Masm32

end start
; 

** Text after 'end start' will not be read by the assembler - no need to comment it out **

; You can safely IGNORE the following lines, they are for use with the RichMasm editor
;OPT_Linker	polink	; use Pelle's linker
;OPT_OUT	TestLib_AB.exe	; call the exe AB, meaning ShowBoxC is not included
;OPT_Tmp2Asm	1	; produce a plain Ascii version of the RichText source
